@@ -10,11 +10,13 @@
 #include <unordered_set>
 #include <vector>
 
-#include "options/options_type.h"
+#include "options/cf_options.h"
+#include "options/db_options.h"
 #include "rocksdb/cache.h"
 #include "rocksdb/compaction_filter.h"
 #include "rocksdb/convenience.h"
 #include "rocksdb/filter_policy.h"
+#include "rocksdb/flush_block_policy.h"
 #include "rocksdb/memtablerep.h"
 #include "rocksdb/merge_operator.h"
 #include "rocksdb/options.h"
@@ -22,11 +24,23 @@
 #include "rocksdb/slice_transform.h"
 #include "rocksdb/table.h"
 #include "rocksdb/utilities/object_registry.h"
-#include "table/block_based/block_based_table_factory.h"
-#include "table/plain/plain_table_factory.h"
+#include "rocksdb/utilities/options_type.h"
 #include "util/string_util.h"
 
 namespace ROCKSDB_NAMESPACE {
+Status ValidateOptions(const DBOptions& db_opts,
+                       const ColumnFamilyOptions& cf_opts) {
+  Status s;
+#ifndef ROCKSDB_LITE
+  auto db_cfg = DBOptionsAsConfigurable(db_opts);
+  auto cf_cfg = CFOptionsAsConfigurable(cf_opts);
+  s = db_cfg->ValidateOptions(db_opts, cf_opts);
+  if (s.ok()) s = cf_cfg->ValidateOptions(db_opts, cf_opts);
+#else
+  s = cf_opts.table_factory->ValidateOptions(db_opts, cf_opts);
+#endif
+  return s;
+}
 
 DBOptions BuildDBOptions(const ImmutableDBOptions& immutable_db_options,
                          const MutableDBOptions& mutable_db_options) {
@@ -61,7 +75,7 @@ DBOptions BuildDBOptions(const ImmutableDBOptions& immutable_db_options,
   options.bytes_per_sync = mutable_db_options.bytes_per_sync;
   options.wal_bytes_per_sync = mutable_db_options.wal_bytes_per_sync;
   options.strict_bytes_per_sync = mutable_db_options.strict_bytes_per_sync;
-  options.max_subcompactions = immutable_db_options.max_subcompactions;
+  options.max_subcompactions = mutable_db_options.max_subcompactions;
   options.max_background_flushes = mutable_db_options.max_background_flushes;
   options.max_log_file_size = immutable_db_options.max_log_file_size;
   options.log_file_time_to_roll = immutable_db_options.log_file_time_to_roll;
@@ -146,6 +160,10 @@ DBOptions BuildDBOptions(const ImmutableDBOptions& immutable_db_options,
   options.file_checksum_gen_factory =
       immutable_db_options.file_checksum_gen_factory;
   options.best_efforts_recovery = immutable_db_options.best_efforts_recovery;
+  options.max_bgerror_resume_count =
+      immutable_db_options.max_bgerror_resume_count;
+  options.bgerror_resume_retry_interval =
+      immutable_db_options.bgerror_resume_retry_interval;
   return options;
 }
 
@@ -203,9 +221,17 @@ ColumnFamilyOptions BuildColumnFamilyOptions(
   cf_opts.compaction_options_universal =
       mutable_cf_options.compaction_options_universal;
 
+  // Blob file related options
+  cf_opts.enable_blob_files = mutable_cf_options.enable_blob_files;
+  cf_opts.min_blob_size = mutable_cf_options.min_blob_size;
+  cf_opts.blob_file_size = mutable_cf_options.blob_file_size;
+  cf_opts.blob_compression_type = mutable_cf_options.blob_compression_type;
+
   // Misc options
   cf_opts.max_sequential_skip_in_iterations =
       mutable_cf_options.max_sequential_skip_in_iterations;
+  cf_opts.check_flush_compaction_key_order =
+      mutable_cf_options.check_flush_compaction_key_order;
   cf_opts.paranoid_file_checks = mutable_cf_options.paranoid_file_checks;
   cf_opts.report_bg_io_stats = mutable_cf_options.report_bg_io_stats;
   cf_opts.compression = mutable_cf_options.compression;
@@ -326,8 +352,8 @@ bool ParseSliceTransform(
   return false;
 }
 
-bool ParseOptionHelper(char* opt_address, const OptionType& opt_type,
-                       const std::string& value) {
+static bool ParseOptionHelper(char* opt_address, const OptionType& opt_type,
+                              const std::string& value) {
   switch (opt_type) {
     case OptionType::kBoolean:
       *reinterpret_cast<bool*>(opt_address) = ParseBoolean("", value);
@@ -462,14 +488,6 @@ bool SerializeSingleOptionHelper(const char* opt_address,
                                           : kNullptrString;
       break;
     }
-    case OptionType::kTableFactory: {
-      const auto* table_factory_ptr =
-          reinterpret_cast<const std::shared_ptr<const TableFactory>*>(
-              opt_address);
-      *value = table_factory_ptr->get() ? table_factory_ptr->get()->Name()
-                                        : kNullptrString;
-      break;
-    }
     case OptionType::kComparator: {
       // it's a const pointer of const Comparator*
       const auto* ptr = reinterpret_cast<const Comparator* const*>(opt_address);
@@ -545,36 +563,29 @@ bool SerializeSingleOptionHelper(const char* opt_address,
   return true;
 }
 
+template <typename T>
+Status ConfigureFromMap(
+    const ConfigOptions& config_options,
+    const std::unordered_map<std::string, std::string>& opt_map,
+    const std::string& option_name, Configurable* config, T* new_opts) {
+  Status s = config->ConfigureFromMap(config_options, opt_map);
+  if (s.ok()) {
+    *new_opts = *(config->GetOptions<T>(option_name));
+  }
+  return s;
+}
+
 Status GetMutableOptionsFromStrings(
     const MutableCFOptions& base_options,
     const std::unordered_map<std::string, std::string>& options_map,
-    Logger* info_log, MutableCFOptions* new_options) {
+    Logger* /*info_log*/, MutableCFOptions* new_options) {
   assert(new_options);
   *new_options = base_options;
   ConfigOptions config_options;
-  for (const auto& o : options_map) {
-    std::string elem;
-    const auto opt_info =
-        OptionTypeInfo::Find(o.first, cf_options_type_info, &elem);
-    if (opt_info == nullptr) {
-      return Status::InvalidArgument("Unrecognized option: " + o.first);
-    } else if (!opt_info->IsMutable()) {
-      return Status::InvalidArgument("Option not changeable: " + o.first);
-    } else if (opt_info->IsDeprecated()) {
-      // log warning when user tries to set a deprecated option but don't fail
-      // the call for compatibility.
-      ROCKS_LOG_WARN(info_log, "%s is a deprecated option and cannot be set",
-                     o.first.c_str());
-    } else {
-      Status s = opt_info->Parse(
-          config_options, elem, o.second,
-          reinterpret_cast<char*>(new_options) + opt_info->mutable_offset_);
-      if (!s.ok()) {
-        return s;
-      }
-    }
-  }
-  return Status::OK();
+  const auto config = CFOptionsAsConfigurable(base_options);
+  return ConfigureFromMap<MutableCFOptions>(config_options, options_map,
+                                            MutableCFOptions::kName(),
+                                            config.get(), new_options);
 }
 
 Status GetMutableDBOptionsFromStrings(
@@ -585,29 +596,10 @@ Status GetMutableDBOptionsFromStrings(
   *new_options = base_options;
   ConfigOptions config_options;
 
-  for (const auto& o : options_map) {
-    try {
-      std::string elem;
-      const auto opt_info =
-          OptionTypeInfo::Find(o.first, db_options_type_info, &elem);
-      if (opt_info == nullptr) {
-        return Status::InvalidArgument("Unrecognized option: " + o.first);
-      } else if (!opt_info->IsMutable()) {
-        return Status::InvalidArgument("Option not changeable: " + o.first);
-      } else {
-        Status s = opt_info->Parse(
-            config_options, elem, o.second,
-            reinterpret_cast<char*>(new_options) + opt_info->mutable_offset_);
-        if (!s.ok()) {
-          return s;
-        }
-      }
-    } catch (std::exception& e) {
-      return Status::InvalidArgument("Error parsing " + o.first + ":" +
-                                     std::string(e.what()));
-    }
-  }
-  return Status::OK();
+  auto config = DBOptionsAsConfigurable(base_options);
+  return ConfigureFromMap<MutableDBOptions>(config_options, options_map,
+                                            MutableDBOptions::kName(),
+                                            config.get(), new_options);
 }
 
 Status StringToMap(const std::string& opts_str,
@@ -650,30 +642,11 @@ Status StringToMap(const std::string& opts_str,
   return Status::OK();
 }
 
-Status GetStringFromStruct(
-    const ConfigOptions& config_options, const void* const opt_ptr,
-    const std::unordered_map<std::string, OptionTypeInfo>& type_info,
-    std::string* opt_string) {
-  assert(opt_string);
-  opt_string->clear();
-  for (const auto iter : type_info) {
-    const auto& opt_info = iter.second;
-    // If the option is no longer used in rocksdb and marked as deprecated,
-    // we skip it in the serialization.
-    if (opt_info.ShouldSerialize()) {
-      const char* opt_addr =
-          reinterpret_cast<const char*>(opt_ptr) + opt_info.offset_;
-      std::string value;
-      Status s =
-          opt_info.Serialize(config_options, iter.first, opt_addr, &value);
-      if (s.ok()) {
-        opt_string->append(iter.first + "=" + value + config_options.delimiter);
-      } else {
-        return s;
-      }
-    }
-  }
-  return Status::OK();
+Status GetStringFromMutableDBOptions(const ConfigOptions& config_options,
+                                     const MutableDBOptions& mutable_opts,
+                                     std::string* opt_string) {
+  auto config = DBOptionsAsConfigurable(mutable_opts);
+  return config->GetOptionString(config_options, opt_string);
 }
 
 Status GetStringFromDBOptions(std::string* opt_string,
@@ -684,11 +657,22 @@ Status GetStringFromDBOptions(std::string* opt_string,
   return GetStringFromDBOptions(config_options, db_options, opt_string);
 }
 
-Status GetStringFromDBOptions(const ConfigOptions& cfg_options,
+Status GetStringFromDBOptions(const ConfigOptions& config_options,
                               const DBOptions& db_options,
                               std::string* opt_string) {
-  return GetStringFromStruct(cfg_options, &db_options, db_options_type_info,
-                             opt_string);
+  assert(opt_string);
+  opt_string->clear();
+  auto config = DBOptionsAsConfigurable(db_options);
+  return config->GetOptionString(config_options, opt_string);
+}
+
+Status GetStringFromMutableCFOptions(const ConfigOptions& config_options,
+                                     const MutableCFOptions& mutable_opts,
+                                     std::string* opt_string) {
+  assert(opt_string);
+  opt_string->clear();
+  const auto config = CFOptionsAsConfigurable(mutable_opts);
+  return config->GetOptionString(config_options, opt_string);
 }
 
 Status GetStringFromColumnFamilyOptions(std::string* opt_string,
@@ -703,8 +687,8 @@ Status GetStringFromColumnFamilyOptions(std::string* opt_string,
 Status GetStringFromColumnFamilyOptions(const ConfigOptions& config_options,
                                         const ColumnFamilyOptions& cf_options,
                                         std::string* opt_string) {
-  return GetStringFromStruct(config_options, &cf_options, cf_options_type_info,
-                             opt_string);
+  const auto config = CFOptionsAsConfigurable(cf_options);
+  return config->GetOptionString(config_options, opt_string);
 }
 
 Status GetStringFromCompressionType(std::string* compression_str,
@@ -718,24 +702,6 @@ Status GetStringFromCompressionType(std::string* compression_str,
   }
 }
 
-static Status ParseDBOption(const ConfigOptions& config_options,
-                            const std::string& name,
-                            const std::string& org_value,
-                            DBOptions* new_options) {
-  const std::string& value = config_options.input_strings_escaped
-                                 ? UnescapeOptionString(org_value)
-                                 : org_value;
-  std::string elem;
-  const auto opt_info = OptionTypeInfo::Find(name, db_options_type_info, &elem);
-  if (opt_info == nullptr) {
-    return Status::InvalidArgument("Unrecognized option DBOptions:", name);
-  } else {
-    return opt_info->Parse(
-        config_options, elem, value,
-        reinterpret_cast<char*>(new_options) + opt_info->offset_);
-  }
-}
-
 Status GetColumnFamilyOptionsFromMap(
     const ColumnFamilyOptions& base_options,
     const std::unordered_map<std::string, std::string>& opts_map,
@@ -754,24 +720,19 @@ Status GetColumnFamilyOptionsFromMap(
     const std::unordered_map<std::string, std::string>& opts_map,
     ColumnFamilyOptions* new_options) {
   assert(new_options);
+
   *new_options = base_options;
-  for (const auto& o : opts_map) {
-    auto s =
-        ParseColumnFamilyOption(config_options, o.first, o.second, new_options);
-    if (!s.ok()) {
-      if (s.IsNotSupported()) {
-        continue;
-      } else if (s.IsInvalidArgument() &&
-                 config_options.ignore_unknown_options) {
-        continue;
-      } else {
-        // Restore "new_options" to the default "base_options".
-        *new_options = base_options;
-        return s;
-      }
-    }
+
+  const auto config = CFOptionsAsConfigurable(base_options);
+  Status s = ConfigureFromMap<ColumnFamilyOptions>(
+      config_options, opts_map, OptionsHelper::kCFOptionsName, config.get(),
+      new_options);
+  // Translate any errors (NotFound, NotSupported, to InvalidArgument
+  if (s.ok() || s.IsInvalidArgument()) {
+    return s;
+  } else {
+    return Status::InvalidArgument(s.getState());
   }
-  return Status::OK();
 }
 
 Status GetColumnFamilyOptionsFromString(
@@ -815,44 +776,18 @@ Status GetDBOptionsFromMap(
     const ConfigOptions& config_options, const DBOptions& base_options,
     const std::unordered_map<std::string, std::string>& opts_map,
     DBOptions* new_options) {
-  return GetDBOptionsFromMapInternal(config_options, base_options, opts_map,
-                                     new_options, nullptr);
-}
-
-Status GetDBOptionsFromMapInternal(
-    const ConfigOptions& config_options, const DBOptions& base_options,
-    const std::unordered_map<std::string, std::string>& opts_map,
-    DBOptions* new_options,
-    std::vector<std::string>* unsupported_options_names) {
   assert(new_options);
   *new_options = base_options;
-  if (unsupported_options_names) {
-    unsupported_options_names->clear();
+  auto config = DBOptionsAsConfigurable(base_options);
+  Status s = ConfigureFromMap<DBOptions>(config_options, opts_map,
+                                         OptionsHelper::kDBOptionsName,
+                                         config.get(), new_options);
+  // Translate any errors (NotFound, NotSupported, to InvalidArgument
+  if (s.ok() || s.IsInvalidArgument()) {
+    return s;
+  } else {
+    return Status::InvalidArgument(s.getState());
   }
-  for (const auto& o : opts_map) {
-    auto s = ParseDBOption(config_options, o.first, o.second, new_options);
-    if (!s.ok()) {
-      if (s.IsNotSupported()) {
-        // If the deserialization of the specified option is not supported
-        // and an output vector of unsupported_options is provided, then
-        // we log the name of the unsupported option and proceed.
-        if (unsupported_options_names != nullptr) {
-          unsupported_options_names->push_back(o.first);
-        }
-        // Note that we still return Status::OK in such case to maintain
-        // the backward compatibility in the old public API defined in
-        // rocksdb/convenience.h
-      } else if (s.IsInvalidArgument() &&
-                 config_options.ignore_unknown_options) {
-        continue;
-      } else {
-        // Restore "new_options" to the default "base_options".
-        *new_options = base_options;
-        return s;
-      }
-    }
-  }
-  return Status::OK();
 }
 
 Status GetDBOptionsFromString(const DBOptions& base_options,
@@ -893,66 +828,37 @@ Status GetOptionsFromString(const Options& base_options,
 Status GetOptionsFromString(const ConfigOptions& config_options,
                             const Options& base_options,
                             const std::string& opts_str, Options* new_options) {
+  ColumnFamilyOptions new_cf_options;
+  std::unordered_map<std::string, std::string> unused_opts;
   std::unordered_map<std::string, std::string> opts_map;
+
+  *new_options = base_options;
   Status s = StringToMap(opts_str, &opts_map);
   if (!s.ok()) {
     return s;
   }
-  DBOptions new_db_options(base_options);
-  ColumnFamilyOptions new_cf_options(base_options);
-  for (const auto& o : opts_map) {
-    if (ParseDBOption(config_options, o.first, o.second, &new_db_options)
-            .ok()) {
-    } else if (ParseColumnFamilyOption(config_options, o.first, o.second,
-                                       &new_cf_options)
-                   .ok()) {
+  auto config = DBOptionsAsConfigurable(base_options);
+  s = config->ConfigureFromMap(config_options, opts_map, &unused_opts);
+
+  if (s.ok()) {
+    DBOptions* new_db_options =
+        config->GetOptions<DBOptions>(OptionsHelper::kDBOptionsName);
+    if (!unused_opts.empty()) {
+      s = GetColumnFamilyOptionsFromMap(config_options, base_options,
+                                        unused_opts, &new_cf_options);
+      if (s.ok()) {
+        *new_options = Options(*new_db_options, new_cf_options);
+      }
     } else {
-      return Status::InvalidArgument("Can't parse option " + o.first);
+      *new_options = Options(*new_db_options, base_options);
     }
   }
-  *new_options = Options(new_db_options, new_cf_options);
-  return Status::OK();
-}
-
-Status GetTableFactoryFromMap(
-    const std::string& factory_name,
-    const std::unordered_map<std::string, std::string>& opt_map,
-    std::shared_ptr<TableFactory>* table_factory, bool ignore_unknown_options) {
-  ConfigOptions
-      config_options;  // Use default for escaped(true) and check (exact)
-  config_options.ignore_unknown_options = ignore_unknown_options;
-  return GetTableFactoryFromMap(config_options, factory_name, opt_map,
-                                table_factory);
-}
-
-Status GetTableFactoryFromMap(
-    const ConfigOptions& config_options, const std::string& factory_name,
-    const std::unordered_map<std::string, std::string>& opt_map,
-    std::shared_ptr<TableFactory>* table_factory) {
-  Status s;
-  if (factory_name == BlockBasedTableFactory().Name()) {
-    BlockBasedTableOptions bbt_opt;
-    s = GetBlockBasedTableOptionsFromMap(
-        config_options, BlockBasedTableOptions(), opt_map, &bbt_opt);
-    if (!s.ok()) {
-      return s;
-    }
-    table_factory->reset(new BlockBasedTableFactory(bbt_opt));
-    return s;
-  } else if (factory_name == PlainTableFactory().Name()) {
-    PlainTableOptions pt_opt;
-    s = GetPlainTableOptionsFromMap(config_options, PlainTableOptions(),
-                                    opt_map, &pt_opt);
-    if (!s.ok()) {
-      return s;
-    }
-    table_factory->reset(new PlainTableFactory(pt_opt));
+  // Translate any errors (NotFound, NotSupported, to InvalidArgument
+  if (s.ok() || s.IsInvalidArgument()) {
     return s;
+  } else {
+    return Status::InvalidArgument(s.getState());
   }
-  // Return OK for not supported table factories as TableFactory
-  // Deserialization is optional.
-  table_factory->reset();
-  return s;
 }
 
 std::unordered_map<std::string, EncodingType>
@@ -1033,18 +939,41 @@ Status OptionTypeInfo::NextToken(const std::string& opts, char delimiter,
 
 Status OptionTypeInfo::Parse(const ConfigOptions& config_options,
                              const std::string& opt_name,
-                             const std::string& opt_value,
-                             char* opt_addr) const {
+                             const std::string& value, void* opt_ptr) const {
   if (IsDeprecated()) {
     return Status::OK();
   }
   try {
+    char* opt_addr = reinterpret_cast<char*>(opt_ptr) + offset_;
+    const std::string& opt_value = config_options.input_strings_escaped
+                                       ? UnescapeOptionString(value)
+                                       : value;
+
     if (opt_addr == nullptr) {
-      return Status::NotFound("Could not find option: ", opt_name);
+      return Status::NotFound("Could not find option", opt_name);
     } else if (parse_func_ != nullptr) {
-      return parse_func_(config_options, opt_name, opt_value, opt_addr);
+      ConfigOptions copy = config_options;
+      copy.invoke_prepare_options = false;
+      return parse_func_(copy, opt_name, opt_value, opt_addr);
     } else if (ParseOptionHelper(opt_addr, type_, opt_value)) {
       return Status::OK();
+    } else if (IsConfigurable()) {
+      // The option is <config>.<name>
+      Configurable* config = AsRawPointer<Configurable>(opt_ptr);
+      if (opt_value.empty()) {
+        return Status::OK();
+      } else if (config == nullptr) {
+        return Status::NotFound("Could not find configurable: ", opt_name);
+      } else {
+        ConfigOptions copy = config_options;
+        copy.ignore_unknown_options = false;
+        copy.invoke_prepare_options = false;
+        if (opt_value.find("=") != std::string::npos) {
+          return config->ConfigureFromString(copy, opt_value);
+        } else {
+          return config->ConfigureOption(copy, opt_name, opt_value);
+        }
+      }
     } else if (IsByName()) {
       return Status::NotSupported("Deserializing the option " + opt_name +
                                   " is not supported");
@@ -1073,11 +1002,10 @@ Status OptionTypeInfo::ParseStruct(
       }
       const auto iter = struct_map->find(map_iter.first);
       if (iter != struct_map->end()) {
-        status =
-            iter->second.Parse(config_options, map_iter.first, map_iter.second,
-                               opt_addr + iter->second.offset_);
+        status = iter->second.Parse(config_options, map_iter.first,
+                                    map_iter.second, opt_addr);
       } else {
-        status = Status::InvalidArgument("Unrecognized option: ",
+        status = Status::InvalidArgument("Unrecognized option",
                                          struct_name + "." + map_iter.first);
       }
     }
@@ -1087,20 +1015,18 @@ Status OptionTypeInfo::ParseStruct(
     const auto opt_info =
         Find(opt_name.substr(struct_name.size() + 1), *struct_map, &elem_name);
     if (opt_info != nullptr) {
-      status = opt_info->Parse(config_options, elem_name, opt_value,
-                               opt_addr + opt_info->offset_);
+      status = opt_info->Parse(config_options, elem_name, opt_value, opt_addr);
     } else {
-      status = Status::InvalidArgument("Unrecognized option: ", opt_name);
+      status = Status::InvalidArgument("Unrecognized option", opt_name);
     }
   } else {
     // This option represents a field in the struct (e.g. field)
     std::string elem_name;
     const auto opt_info = Find(opt_name, *struct_map, &elem_name);
     if (opt_info != nullptr) {
-      status = opt_info->Parse(config_options, elem_name, opt_value,
-                               opt_addr + opt_info->offset_);
+      status = opt_info->Parse(config_options, elem_name, opt_value, opt_addr);
     } else {
-      status = Status::InvalidArgument("Unrecognized option: ",
+      status = Status::InvalidArgument("Unrecognized option",
                                        struct_name + "." + opt_name);
     }
   }
@@ -1109,18 +1035,30 @@ Status OptionTypeInfo::ParseStruct(
 
 Status OptionTypeInfo::Serialize(const ConfigOptions& config_options,
                                  const std::string& opt_name,
-                                 const char* opt_addr,
+                                 const void* const opt_ptr,
                                  std::string* opt_value) const {
   // If the option is no longer used in rocksdb and marked as deprecated,
   // we skip it in the serialization.
-  if (opt_addr != nullptr && ShouldSerialize()) {
-    if (serialize_func_ != nullptr) {
-      return serialize_func_(config_options, opt_name, opt_addr, opt_value);
-    } else if (!SerializeSingleOptionHelper(opt_addr, type_, opt_value)) {
-      return Status::InvalidArgument("Cannot serialize option: ", opt_name);
+  const char* opt_addr = reinterpret_cast<const char*>(opt_ptr) + offset_;
+  if (opt_addr == nullptr || IsDeprecated()) {
+    return Status::OK();
+  } else if (IsEnabled(OptionTypeFlags::kDontSerialize)) {
+    return Status::NotSupported("Cannot serialize option: ", opt_name);
+  } else if (serialize_func_ != nullptr) {
+    return serialize_func_(config_options, opt_name, opt_addr, opt_value);
+  } else if (SerializeSingleOptionHelper(opt_addr, type_, opt_value)) {
+    return Status::OK();
+  } else if (IsConfigurable()) {
+    const Configurable* config = AsRawPointer<Configurable>(opt_ptr);
+    if (config != nullptr) {
+      ConfigOptions embedded = config_options;
+      embedded.delimiter = ";";
+      *opt_value = config->ToString(embedded);
     }
+    return Status::OK();
+  } else {
+    return Status::InvalidArgument("Cannot serialize option: ", opt_name);
   }
-  return Status::OK();
 }
 
 Status OptionTypeInfo::SerializeStruct(
@@ -1141,8 +1079,7 @@ Status OptionTypeInfo::SerializeStruct(
       std::string single;
       const auto& opt_info = iter.second;
       if (opt_info.ShouldSerialize()) {
-        status = opt_info.Serialize(embedded, iter.first,
-                                    opt_addr + opt_info.offset_, &single);
+        status = opt_info.Serialize(embedded, iter.first, opt_addr, &single);
         if (!status.ok()) {
           return status;
         } else {
@@ -1157,20 +1094,19 @@ Status OptionTypeInfo::SerializeStruct(
     const auto opt_info =
         Find(opt_name.substr(struct_name.size() + 1), *struct_map, &elem_name);
     if (opt_info != nullptr) {
-      status = opt_info->Serialize(config_options, elem_name,
-                                   opt_addr + opt_info->offset_, value);
+      status = opt_info->Serialize(config_options, elem_name, opt_addr, value);
     } else {
-      status = Status::InvalidArgument("Unrecognized option: ", opt_name);
+      status = Status::InvalidArgument("Unrecognized option", opt_name);
     }
   } else {
     // This option represents a field in the struct (e.g. field)
     std::string elem_name;
     const auto opt_info = Find(opt_name, *struct_map, &elem_name);
     if (opt_info == nullptr) {
-      status = Status::InvalidArgument("Unrecognized option: ", opt_name);
+      status = Status::InvalidArgument("Unrecognized option", opt_name);
     } else if (opt_info->ShouldSerialize()) {
       status = opt_info->Serialize(config_options, opt_name + "." + elem_name,
-                                   opt_addr + opt_info->offset_, value);
+                                   opt_addr, value);
     }
   }
   return status;
@@ -1241,11 +1177,15 @@ static bool AreOptionsEqual(OptionType type, const char* this_offset,
 
 bool OptionTypeInfo::AreEqual(const ConfigOptions& config_options,
                               const std::string& opt_name,
-                              const char* this_addr, const char* that_addr,
+                              const void* const this_ptr,
+                              const void* const that_ptr,
                               std::string* mismatch) const {
-  if (!config_options.IsCheckEnabled(GetSanityLevel())) {
+  auto level = GetSanityLevel();
+  if (!config_options.IsCheckEnabled(level)) {
     return true;  // If the sanity level is not being checked, skip it
   }
+  const auto this_addr = reinterpret_cast<const char*>(this_ptr) + offset_;
+  const auto that_addr = reinterpret_cast<const char*>(that_ptr) + offset_;
   if (this_addr == nullptr || that_addr == nullptr) {
     if (this_addr == that_addr) {
       return true;
@@ -1257,6 +1197,27 @@ bool OptionTypeInfo::AreEqual(const ConfigOptions& config_options,
     }
   } else if (AreOptionsEqual(type_, this_addr, that_addr)) {
     return true;
+  } else if (IsConfigurable()) {
+    const auto* this_config = AsRawPointer<Configurable>(this_ptr);
+    const auto* that_config = AsRawPointer<Configurable>(that_ptr);
+    if (this_config == that_config) {
+      return true;
+    } else if (this_config != nullptr && that_config != nullptr) {
+      std::string bad_name;
+      bool matches;
+      if (level < config_options.sanity_level) {
+        ConfigOptions copy = config_options;
+        copy.sanity_level = level;
+        matches = this_config->AreEquivalent(copy, that_config, &bad_name);
+      } else {
+        matches =
+            this_config->AreEquivalent(config_options, that_config, &bad_name);
+      }
+      if (!matches) {
+        *mismatch = opt_name + "." + bad_name;
+      }
+      return matches;
+    }
   }
   if (mismatch->empty()) {
     *mismatch = opt_name;
@@ -1277,9 +1238,8 @@ bool OptionTypeInfo::StructsAreEqual(
     for (const auto& iter : *struct_map) {
       const auto& opt_info = iter.second;
 
-      matches = opt_info.AreEqual(config_options, iter.first,
-                                  this_addr + opt_info.offset_,
-                                  that_addr + opt_info.offset_, &result);
+      matches = opt_info.AreEqual(config_options, iter.first, this_addr,
+                                  that_addr, &result);
       if (!matches) {
         *mismatch = struct_name + "." + result;
         return false;
@@ -1294,9 +1254,8 @@ bool OptionTypeInfo::StructsAreEqual(
     if (opt_info == nullptr) {
       *mismatch = opt_name;
       matches = false;
-    } else if (!opt_info->AreEqual(config_options, elem_name,
-                                   this_addr + opt_info->offset_,
-                                   that_addr + opt_info->offset_, &result)) {
+    } else if (!opt_info->AreEqual(config_options, elem_name, this_addr,
+                                   that_addr, &result)) {
       matches = false;
       *mismatch = struct_name + "." + result;
     }
@@ -1308,9 +1267,8 @@ bool OptionTypeInfo::StructsAreEqual(
     if (opt_info == nullptr) {
       *mismatch = struct_name + "." + opt_name;
       matches = false;
-    } else if (!opt_info->AreEqual(config_options, elem_name,
-                                   this_addr + opt_info->offset_,
-                                   that_addr + opt_info->offset_, &result)) {
+    } else if (!opt_info->AreEqual(config_options, elem_name, this_addr,
+                                   that_addr, &result)) {
       matches = false;
       *mismatch = struct_name + "." + result;
     }
@@ -1318,14 +1276,34 @@ bool OptionTypeInfo::StructsAreEqual(
   return matches;
 }
 
+bool MatchesOptionsTypeFromMap(
+    const ConfigOptions& config_options,
+    const std::unordered_map<std::string, OptionTypeInfo>& type_map,
+    const void* const this_ptr, const void* const that_ptr,
+    std::string* mismatch) {
+  for (auto& pair : type_map) {
+    // We skip checking deprecated variables as they might
+    // contain random values since they might not be initialized
+    if (config_options.IsCheckEnabled(pair.second.GetSanityLevel())) {
+      if (!pair.second.AreEqual(config_options, pair.first, this_ptr, that_ptr,
+                                mismatch) &&
+          !pair.second.AreEqualByName(config_options, pair.first, this_ptr,
+                                      that_ptr)) {
+        return false;
+      }
+    }
+  }
+  return true;
+}
+
 bool OptionTypeInfo::AreEqualByName(const ConfigOptions& config_options,
                                     const std::string& opt_name,
-                                    const char* this_addr,
-                                    const char* that_addr) const {
+                                    const void* const this_ptr,
+                                    const void* const that_ptr) const {
   if (IsByName()) {
     std::string that_value;
-    if (Serialize(config_options, opt_name, that_addr, &that_value).ok()) {
-      return AreEqualByName(config_options, opt_name, this_addr, that_value);
+    if (Serialize(config_options, opt_name, that_ptr, &that_value).ok()) {
+      return AreEqualByName(config_options, opt_name, this_ptr, that_value);
     }
   }
   return false;
@@ -1333,12 +1311,12 @@ bool OptionTypeInfo::AreEqualByName(const ConfigOptions& config_options,
 
 bool OptionTypeInfo::AreEqualByName(const ConfigOptions& config_options,
                                     const std::string& opt_name,
-                                    const char* opt_addr,
+                                    const void* const opt_ptr,
                                     const std::string& that_value) const {
   std::string this_value;
   if (!IsByName()) {
     return false;
-  } else if (!Serialize(config_options, opt_name, opt_addr, &this_value).ok()) {
+  } else if (!Serialize(config_options, opt_name, opt_ptr, &this_value).ok()) {
     return false;
   } else if (IsEnabled(OptionVerificationType::kByNameAllowFromNull)) {
     if (that_value == kNullptrString) {
@@ -1366,7 +1344,8 @@ const OptionTypeInfo* OptionTypeInfo::Find(
       auto siter =
           opt_map.find(opt_name.substr(0, idx));  // Look for the short name
       if (siter != opt_map.end()) {               // We found the short name
-        if (siter->second.IsStruct()) {           // If the object is a struct
+        if (siter->second.IsStruct() ||           // If the object is a struct
+            siter->second.IsConfigurable()) {     // or a Configurable
           *elem_name = opt_name.substr(idx + 1);  // Return the rest
           return &(siter->second);  // Return the contents of the iterator
         }
