@@ -500,7 +500,7 @@ void ReadKeys(uint64_t num, uint32_t batch_size) {
   for (uint64_t i = 0; i < num; ++i) {
     keys.push_back(2 * i);
   }
-  std::random_shuffle(keys.begin(), keys.end());
+  RandomShuffle(keys.begin(), keys.end());
 
   PinnableSlice value;
   // Assume only the fast path is triggered
