@@ -26,6 +26,8 @@
 #include "rocksdb/transaction_log.h"
 #include "rocksdb/utilities/checkpoint.h"
 #include "test_util/sync_point.h"
+#include "util/cast_util.h"
+#include "util/file_checksum_helper.h"
 
 namespace ROCKSDB_NAMESPACE {
 
@@ -118,7 +120,9 @@ Status CheckpointImpl::CreateCheckpoint(const std::string& checkpoint_dir,
                                                 IOOptions(), nullptr);
         } /* link_file_cb */,
         [&](const std::string& src_dirname, const std::string& fname,
-            uint64_t size_limit_bytes, FileType) {
+            uint64_t size_limit_bytes, FileType,
+            const std::string& /* checksum_func_name */,
+            const std::string& /* checksum_val */) {
           ROCKS_LOG_INFO(db_options.info_log, "Copying %s", fname.c_str());
           return CopyFile(db_->GetFileSystem(), src_dirname + fname,
                           full_private_path + fname, size_limit_bytes,
@@ -168,14 +172,16 @@ Status CheckpointImpl::CreateCustomCheckpoint(
     std::function<Status(const std::string& src_dirname,
                          const std::string& src_fname, FileType type)>
         link_file_cb,
-    std::function<Status(const std::string& src_dirname,
-                         const std::string& src_fname,
-                         uint64_t size_limit_bytes, FileType type)>
+    std::function<Status(
+        const std::string& src_dirname, const std::string& src_fname,
+        uint64_t size_limit_bytes, FileType type,
+        const std::string& checksum_func_name, const std::string& checksum_val)>
         copy_file_cb,
     std::function<Status(const std::string& fname, const std::string& contents,
                          FileType type)>
         create_file_cb,
-    uint64_t* sequence_number, uint64_t log_size_for_flush) {
+    uint64_t* sequence_number, uint64_t log_size_for_flush,
+    bool get_live_table_checksum) {
   Status s;
   std::vector<std::string> live_files;
   uint64_t manifest_file_size = 0;
@@ -245,6 +251,8 @@ Status CheckpointImpl::CreateCustomCheckpoint(
     TEST_SYNC_POINT("CheckpointImpl::CreateCheckpoint:SavedLiveFiles2");
     db_->FlushWAL(false /* sync */);
   }
+  TEST_SYNC_POINT("CheckpointImpl::CreateCustomCheckpoint:AfterGetLive1");
+  TEST_SYNC_POINT("CheckpointImpl::CreateCustomCheckpoint:AfterGetLive2");
   // if we have more than one column family, we need to also get WAL files
   if (s.ok()) {
     s = db_->GetSortedWalFiles(live_wal_files);
@@ -255,12 +263,17 @@ Status CheckpointImpl::CreateCustomCheckpoint(
 
   size_t wal_size = live_wal_files.size();
 
-  // copy/hard link live_files
+  // process live files, non-table files first
   std::string manifest_fname, current_fname;
-  for (size_t i = 0; s.ok() && i < live_files.size(); ++i) {
+  // record table files for processing next
+  std::vector<std::pair<std::string, uint64_t>> live_table_files;
+  for (auto& live_file : live_files) {
+    if (!s.ok()) {
+      break;
+    }
     uint64_t number;
     FileType type;
-    bool ok = ParseFileName(live_files[i], &number, &type);
+    bool ok = ParseFileName(live_file, &number, &type);
     if (!ok) {
       s = Status::Corruption("Can't parse file name. This is very bad");
       break;
@@ -268,33 +281,80 @@ Status CheckpointImpl::CreateCustomCheckpoint(
     // we should only get sst, options, manifest and current files here
     assert(type == kTableFile || type == kDescriptorFile ||
            type == kCurrentFile || type == kOptionsFile);
-    assert(live_files[i].size() > 0 && live_files[i][0] == '/');
+    assert(live_file.size() > 0 && live_file[0] == '/');
     if (type == kCurrentFile) {
       // We will craft the current file manually to ensure it's consistent with
       // the manifest number. This is necessary because current's file contents
       // can change during checkpoint creation.
-      current_fname = live_files[i];
+      current_fname = live_file;
       continue;
     } else if (type == kDescriptorFile) {
-      manifest_fname = live_files[i];
+      manifest_fname = live_file;
     }
-    std::string src_fname = live_files[i];
+    if (type != kTableFile) {
+      // copy non-table files here
+      // * if it's kDescriptorFile, limit the size to manifest_file_size
+      s = copy_file_cb(db_->GetName(), live_file,
+                       (type == kDescriptorFile) ? manifest_file_size : 0, type,
+                       kUnknownFileChecksumFuncName, kUnknownFileChecksum);
+    } else {
+      // process table files below
+      live_table_files.push_back(make_pair(live_file, number));
+    }
+  }
+
+  // get checksum info for table files
+  // get table file checksums if get_live_table_checksum is true
+  std::unique_ptr<FileChecksumList> checksum_list;
+
+  if (s.ok() && get_live_table_checksum) {
+    checksum_list.reset(NewFileChecksumList());
+    // should succeed even without checksum info present, else manifest
+    // is corrupt
+    s = GetFileChecksumsFromManifest(db_->GetEnv(),
+                                     db_->GetName() + manifest_fname,
+                                     manifest_file_size, checksum_list.get());
+  }
+
+  // copy/hard link live table files
+  for (auto& ltf : live_table_files) {
+    if (!s.ok()) {
+      break;
+    }
+    std::string& src_fname = ltf.first;
+    uint64_t number = ltf.second;
 
     // rules:
-    // * if it's kTableFile, then it's shared
-    // * if it's kDescriptorFile, limit the size to manifest_file_size
-    // * always copy if cross-device link
-    if ((type == kTableFile) && same_fs) {
-      s = link_file_cb(db_->GetName(), src_fname, type);
+    // * for kTableFile, attempt hard link instead of copy.
+    // * but can't hard link across filesystems.
+    if (same_fs) {
+      s = link_file_cb(db_->GetName(), src_fname, kTableFile);
       if (s.IsNotSupported()) {
         same_fs = false;
         s = Status::OK();
       }
     }
-    if ((type != kTableFile) || (!same_fs)) {
-      s = copy_file_cb(db_->GetName(), src_fname,
-                       (type == kDescriptorFile) ? manifest_file_size : 0,
-                       type);
+    if (!same_fs) {
+      std::string checksum_name = kUnknownFileChecksumFuncName;
+      std::string checksum_value = kUnknownFileChecksum;
+
+      // we ignore the checksums either they are not required or we failed to
+      // obtain the checksum lsit for old table files that have no file
+      // checksums
+      if (get_live_table_checksum) {
+        // find checksum info for table files
+        Status search = checksum_list->SearchOneFileChecksum(
+            number, &checksum_value, &checksum_name);
+
+        // could be a legacy file lacking checksum info. overall OK if
+        // not found
+        if (!search.ok()) {
+          assert(checksum_name == kUnknownFileChecksumFuncName);
+          assert(checksum_value == kUnknownFileChecksum);
+        }
+      }
+      s = copy_file_cb(db_->GetName(), src_fname, 0, kTableFile, checksum_name,
+                       checksum_value);
     }
   }
   if (s.ok() && !current_fname.empty() && !manifest_fname.empty()) {
@@ -313,7 +373,8 @@ Status CheckpointImpl::CreateCustomCheckpoint(
          live_wal_files[i]->LogNumber() >= min_log_num)) {
       if (i + 1 == wal_size) {
         s = copy_file_cb(db_options.wal_dir, live_wal_files[i]->PathName(),
-                         live_wal_files[i]->SizeFileBytes(), kLogFile);
+                         live_wal_files[i]->SizeFileBytes(), kLogFile,
+                         kUnknownFileChecksumFuncName, kUnknownFileChecksum);
         break;
       }
       if (same_fs) {
@@ -327,7 +388,8 @@ Status CheckpointImpl::CreateCustomCheckpoint(
       }
       if (!same_fs) {
         s = copy_file_cb(db_options.wal_dir, live_wal_files[i]->PathName(), 0,
-                         kLogFile);
+                         kLogFile, kUnknownFileChecksumFuncName,
+                         kUnknownFileChecksum);
       }
     }
   }
@@ -340,7 +402,7 @@ Status CheckpointImpl::CreateCustomCheckpoint(
 Status CheckpointImpl::ExportColumnFamily(
     ColumnFamilyHandle* handle, const std::string& export_dir,
     ExportImportFilesMetaData** metadata) {
-  auto cfh = reinterpret_cast<ColumnFamilyHandleImpl*>(handle);
+  auto cfh = static_cast_with_check<ColumnFamilyHandleImpl>(handle);
   const auto cf_name = cfh->GetName();
   const auto db_options = db_->GetDBOptions();
 
